@@ -2,7 +2,7 @@
  *
  *  Name:       convert.c
  *
- *  Version:    2.1-1
+ *  Version:    2.3-1
  *
  *  Purpose:    Type conversions for RNetCDF
  *
@@ -125,7 +125,7 @@ R_nc_length_sexp (SEXP count)
       length *= rcount[ii]; 
     }
     if (!R_FINITE (length)) {
-      R_nc_error ("Non-finite length in R_nc_length_sexp");
+      error ("Non-finite length in R_nc_length_sexp");
     }
   } else if (isInteger (count)) {
     icount = INTEGER (count);
@@ -133,11 +133,11 @@ R_nc_length_sexp (SEXP count)
       if (icount[ii] != NA_INTEGER) {
         length *= icount[ii];
       } else {
-        R_nc_error ("Missing value in R_nc_length_sexp");
+        error ("Missing value in R_nc_length_sexp");
       }
     }
   } else if (!isNull (count)) {
-    R_nc_error ("Unsupported type in R_nc_length_sexp");
+    error ("Unsupported type in R_nc_length_sexp");
   }
 
   return (length);
@@ -149,22 +149,23 @@ R_nc_allocArray (SEXPTYPE type, int ndims, const size_t *ccount) {
   SEXP result, rdim;
   int *intp, ii, jj;
   if (ndims > 0) {
-    rdim = R_nc_protect( allocVector (INTSXP, ndims));
+    rdim = PROTECT( allocVector (INTSXP, ndims));
     intp = INTEGER (rdim);
     for ( ii=0, jj=ndims-1; ii<ndims; ii++, jj-- ) {
       if (ccount[jj] <= INT_MAX) {
         intp[ii] = ccount[jj];
       } else {
-        RERROR ("R array dimension cannot exceed range of type int");
+        error ("R array dimension cannot exceed range of type int");
       }
     }
-    result = R_nc_protect (allocArray (type, rdim));
+    result = allocArray (type, rdim);
+    UNPROTECT(1);
   } else if (ndims == 0) {
     /* R scalar with no dimensions */
-    result = R_nc_protect (allocVector (type, 1));
+    result = allocVector (type, 1);
   } else {
     /* R vector of length ccount[0] without a dimension attribute */
-    result = R_nc_protect (allocVector (type, ccount[0]));
+    result = allocVector (type, ccount[0]);
   }
   return result;
 }
@@ -193,8 +194,8 @@ R_nc_strsxp_char (SEXP rstr, int ndim, const size_t *xdim)
     strlen = xdim[0];
     cnt = 1;
   }
-  if (xlength (rstr) < cnt) {
-    RERROR (RNC_EDATALEN);
+  if ((size_t) xlength (rstr) < cnt) {
+    error (RNC_EDATALEN);
   }
   carr = R_alloc (cnt*strlen, sizeof (char));
   for (ii=0, thisstr=carr; ii<cnt; ii++, thisstr+=strlen) {
@@ -204,18 +205,20 @@ R_nc_strsxp_char (SEXP rstr, int ndim, const size_t *xdim)
 }
 
 
-static void
+static SEXP
 R_nc_char_strsxp_init (R_nc_buf *io)
 {
   if (io->ndim > 0) {
-    io->rxp = R_nc_allocArray (STRSXP, (io->ndim)-1, io->xdim);
+    io->rxp = PROTECT(R_nc_allocArray (STRSXP, (io->ndim)-1, io->xdim));
   } else {
     /* Single character or string */
-    io->rxp = R_nc_allocArray (STRSXP, 0, io->xdim);
+    io->rxp = PROTECT(R_nc_allocArray (STRSXP, 0, io->xdim));
   }
   if (!io->cbuf) {
     io->cbuf = R_alloc (R_nc_length (io->ndim, io->xdim), sizeof (char));
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -253,21 +256,23 @@ R_nc_raw_char (SEXP rarr, int ndim, const size_t *xdim)
 {
   size_t cnt;
   cnt = R_nc_length (ndim, xdim);
-  if (xlength (rarr) < cnt) {
-    RERROR (RNC_EDATALEN);
+  if ((size_t) xlength (rarr) < cnt) {
+    error (RNC_EDATALEN);
   }
   return (const char *) RAW (rarr);
 }
 
 
-static void
+static SEXP
 R_nc_char_raw_init (R_nc_buf *io)
 {
-  io->rxp = R_nc_allocArray (RAWSXP, io->ndim, io->xdim);
+  io->rxp = PROTECT(R_nc_allocArray (RAWSXP, io->ndim, io->xdim));
   io->rbuf = RAW (io->rxp);
   if (!io->cbuf) {
     io->cbuf = io->rbuf;
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -287,8 +292,8 @@ R_nc_strsxp_str (SEXP rstr, int ndim, const size_t *xdim)
   size_t ii, cnt;
   const char **cstr;
   cnt = R_nc_length (ndim, xdim);
-  if (xlength (rstr) < cnt) {
-    RERROR (RNC_EDATALEN);
+  if ((size_t) xlength (rstr) < cnt) {
+    error (RNC_EDATALEN);
   }
   cstr = (const char **) R_alloc (cnt, sizeof(size_t));
   for (ii=0; ii<cnt; ii++) {
@@ -298,13 +303,15 @@ R_nc_strsxp_str (SEXP rstr, int ndim, const size_t *xdim)
 }
 
 
-static void
+static SEXP
 R_nc_str_strsxp_init (R_nc_buf *io)
 {
-  io->rxp = R_nc_allocArray (STRSXP, io->ndim, io->xdim);
+  io->rxp = PROTECT(R_nc_allocArray (STRSXP, io->ndim, io->xdim));
   if (!io->cbuf) {
     io->cbuf = R_alloc (xlength (io->rxp), sizeof(size_t));
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -367,8 +374,8 @@ FUN (SEXP rv, int ndim, const size_t *xdim, \
   OTYPE fillval, *out; \
   in = (ITYPE *) IFUN (rv); \
   cnt = R_nc_length (ndim, xdim); \
-  if (xlength (rv) < cnt) { \
-    RERROR (RNC_EDATALEN); \
+  if ((size_t) xlength (rv) < cnt) { \
+    error (RNC_EDATALEN); \
   } \
   if (fill || scale || add || (NCITYPE != NCOTYPE)) { \
     out = (OTYPE *) R_alloc (cnt, sizeof(OTYPE)); \
@@ -387,9 +394,11 @@ FUN (SEXP rv, int ndim, const size_t *xdim, \
   } \
   if (fill) { \
     if (fillsize != sizeof(OTYPE)) { \
-      R_nc_error ("Size of fill value does not match output type"); \
+      error ("Size of fill value does not match output type"); \
     } \
     fillval = *fill; \
+  } else { \
+    fillval = 0; \
   } \
   for (ii=0; ii<cnt; ii++) { \
     if (NATEST(in[ii])) { \
@@ -410,9 +419,9 @@ FUN (SEXP rv, int ndim, const size_t *xdim, \
     } \
   } \
   if ( erange ) { \
-    R_nc_error (nc_strerror (NC_ERANGE)); \
+    error (nc_strerror (NC_ERANGE)); \
   } else if ( efill ) { \
-    R_nc_error ("NA values sent to netcdf without conversion to fill value"); \
+    error ("NA values sent to netcdf without conversion to fill value"); \
   } \
   return out; \
 }
@@ -433,13 +442,18 @@ R_NC_R2C_NUM(R_nc_r2c_int_ll, NC_INT, int, INTEGER, NC_INT64, long long, \
   R_NC_ISNA_INT, R_NC_RANGE_NONE, , R_NC_RANGE_NONE, )
 R_NC_R2C_NUM(R_nc_r2c_int_ull, NC_INT, int, INTEGER, NC_UINT64, unsigned long long, \
   R_NC_ISNA_INT, R_NC_RANGE_MIN, 0, R_NC_RANGE_NONE, )
-/* Assume int and size_t are different types */
-R_NC_R2C_NUM(R_nc_r2c_int_size, NC_INT, int, INTEGER, NC_UINT64, size_t, \
-  R_NC_ISNA_INT, R_NC_RANGE_MIN, 0, R_NC_RANGE_NONE, )
 R_NC_R2C_NUM(R_nc_r2c_int_float, NC_INT, int, INTEGER, NC_FLOAT, float, \
   R_NC_ISNA_INT, R_NC_RANGE_NONE, , R_NC_RANGE_NONE, )
 R_NC_R2C_NUM(R_nc_r2c_int_dbl, NC_INT, int, INTEGER, NC_DOUBLE, double, \
   R_NC_ISNA_INT, R_NC_RANGE_NONE, , R_NC_RANGE_NONE, )
+/* Only convert non-negative values to size_t */
+#if SIZEOF_INT > SIZEOF_SIZE_T
+R_NC_R2C_NUM(R_nc_r2c_int_size, NC_INT, int, INTEGER, NC_NAT, size_t, \
+  R_NC_ISNA_INT, R_NC_RANGE_MIN, 0, R_NC_RANGE_MAX, SIZE_MAX)
+#else
+R_NC_R2C_NUM(R_nc_r2c_int_size, NC_INT, int, INTEGER, NC_NAT, size_t, \
+  R_NC_ISNA_INT, R_NC_RANGE_MIN, 0, R_NC_RANGE_NONE, )
+#endif
 
 R_NC_R2C_NUM(R_nc_r2c_dbl_schar, NC_DOUBLE, double, REAL, NC_BYTE, signed char, \
   R_NC_ISNA_REAL, R_NC_RANGE_MIN, SCHAR_MIN, R_NC_RANGE_MAX, SCHAR_MAX)
@@ -457,12 +471,13 @@ R_NC_R2C_NUM(R_nc_r2c_dbl_ll, NC_DOUBLE, double, REAL, NC_INT64, long long, \
   R_NC_ISNA_REAL, R_NC_RANGE_MIN, LLONG_MIN_DBL, R_NC_RANGE_MAX, LLONG_MAX_DBL)
 R_NC_R2C_NUM(R_nc_r2c_dbl_ull, NC_DOUBLE, double, REAL, NC_UINT64, unsigned long long, \
   R_NC_ISNA_REAL, R_NC_RANGE_MIN, 0, R_NC_RANGE_MAX, ULLONG_MAX_DBL)
-R_NC_R2C_NUM(R_nc_r2c_dbl_size, NC_DOUBLE, double, REAL, NC_UINT64, size_t, \
-  R_NC_ISNA_REAL, R_NC_RANGE_MIN, 0, R_NC_RANGE_MAX, SIZE_MAX_DBL)
 R_NC_R2C_NUM(R_nc_r2c_dbl_float, NC_DOUBLE, double, REAL, NC_FLOAT, float, \
   R_NC_ISNA_REAL, R_NC_RANGE_MIN, -FLT_MAX, R_NC_RANGE_MAX, FLT_MAX)
 R_NC_R2C_NUM(R_nc_r2c_dbl_dbl, NC_DOUBLE, double, REAL, NC_DOUBLE, double, \
   R_NC_ISNA_REAL, R_NC_RANGE_NONE, , R_NC_RANGE_NONE, )
+/* Only convert non-negative values to size_t */
+R_NC_R2C_NUM(R_nc_r2c_dbl_size, NC_DOUBLE, double, REAL, NC_NAT, size_t, \
+  R_NC_ISNA_REAL, R_NC_RANGE_MIN, 0, R_NC_RANGE_MAX, SIZE_MAX_DBL)
 
 /* bit64 is treated by R as signed long long,
    but we may need to store unsigned long long,
@@ -489,16 +504,13 @@ R_NC_R2C_NUM(R_nc_r2c_bit64_float, NC_INT64, long long, REAL, NC_FLOAT, float, \
   R_NC_ISNA_BIT64, R_NC_RANGE_MIN, -FLT_MAX, R_NC_RANGE_MAX, FLT_MAX)
 R_NC_R2C_NUM(R_nc_r2c_bit64_dbl, NC_INT64, long long, REAL, NC_DOUBLE, double, \
   R_NC_ISNA_BIT64, R_NC_RANGE_NONE, , R_NC_RANGE_NONE, )
-#if LLONG_MAX > SIZE_MAX
-/* size_t is smaller than unsigned long long.
-   Only allow positive values of bit64
- */
+/* Assume bit64 can represent size of any object without wrapping */
+#if SIZEOF_LONG_LONG > SIZEOF_SIZE_T
 R_NC_R2C_NUM(R_nc_r2c_bit64_size, NC_INT64, long long, REAL, NC_NAT, size_t, \
   R_NC_ISNA_BIT64, R_NC_RANGE_MIN, 0, R_NC_RANGE_MAX, SIZE_MAX)
 #else
-/* Allow wrapping from negative bit64 to positive size_t */
 R_NC_R2C_NUM(R_nc_r2c_bit64_size, NC_INT64, long long, REAL, NC_NAT, size_t, \
-  R_NC_ISNA_BIT64, R_NC_RANGE_NONE, , R_NC_RANGE_MAX, SIZE_MAX)
+  R_NC_ISNA_BIT64, R_NC_RANGE_MIN, 0, R_NC_RANGE_NONE, )
 #endif
 
 /* Allocate memory for reading a netcdf variable slice
@@ -507,14 +519,16 @@ R_NC_R2C_NUM(R_nc_r2c_bit64_size, NC_INT64, long long, REAL, NC_NAT, size_t, \
    On output, the R_nc_buf structure contains an allocated SEXP and a pointer to its data.
  */
 #define R_NC_C2R_NUM_INIT(FUN, SEXPTYPE, OFUN) \
-static void \
+static SEXP \
 FUN (R_nc_buf *io) \
 { \
-  io->rxp = R_nc_allocArray (SEXPTYPE, io->ndim, io->xdim); \
+  io->rxp = PROTECT(R_nc_allocArray (SEXPTYPE, io->ndim, io->xdim)); \
   io->rbuf = OFUN (io->rxp); \
   if (!io->cbuf) { \
     io->cbuf = io->rbuf; \
   } \
+  UNPROTECT(1); \
+  return io->rxp; \
 }
 
 R_NC_C2R_NUM_INIT(R_nc_c2r_int_init, INTSXP, INTEGER)
@@ -541,10 +555,12 @@ FUN (R_nc_buf *io) \
   in = (ITYPE *) io->cbuf; \
   out = (OTYPE *) io->rbuf; \
   if ((io->fill || io->min || io->max ) && io->fillsize != sizeof(ITYPE)) { \
-    R_nc_error ("Size of fill value does not match input type"); \
+    error ("Size of fill value does not match input type"); \
   } \
   if (io->fill) { \
     fillval = *((ITYPE *) io->fill); \
+  } else { \
+    fillval = 0; \
   } \
   if (io->min) { \
     minval = *((ITYPE *) io->min); \
@@ -647,10 +663,12 @@ FUN (R_nc_buf *io) \
     offset = 0.0; \
   } \
   if ((io->fill || io->min || io->max) && io->fillsize != sizeof(ITYPE)) { \
-    R_nc_error ("Size of fill value does not match input type"); \
+    error ("Size of fill value does not match input type"); \
   } \
   if (io->fill) { \
     fillval = *((ITYPE *) io->fill); \
+  } else { \
+    fillval = 0; \
   } \
   if (io->min) { \
     minval = *((ITYPE *) io->min); \
@@ -715,8 +733,8 @@ R_nc_vecsxp_vlen (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim
   SEXP item;
 
   cnt = R_nc_length (ndim, xdim);
-  if (xlength (rv) < cnt) {
-    RERROR (RNC_EDATALEN);
+  if ((size_t) xlength (rv) < cnt) {
+    error (RNC_EDATALEN);
   }
 
   R_nc_check (nc_inq_user_type (ncid, xtype, NULL, NULL, &basetype, NULL, NULL));
@@ -760,13 +778,15 @@ R_nc_vecsxp_vlen (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim
      and the C buffer is an array of pointers which are allocated by netcdf
      when reading from the variable (and which must be freed later by netcdf).
  */
-static void
+static SEXP
 R_nc_vlen_vecsxp_init (R_nc_buf *io)
 {
-  io->rxp = R_nc_allocArray (VECSXP, io->ndim, io->xdim);
+  io->rxp = PROTECT(R_nc_allocArray (VECSXP, io->ndim, io->xdim));
   if (!io->cbuf) {
     io->cbuf = R_alloc (xlength (io->rxp), sizeof(nc_vlen_t));
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -782,16 +802,20 @@ R_nc_vlen_vecsxp (R_nc_buf *io)
   nc_type basetype;
   nc_vlen_t *vbuf;
   R_nc_buf tmpio;
+  SEXP tmprxp;
 
   vbuf = io->cbuf;
   cnt = xlength (io->rxp);
   R_nc_check (nc_inq_user_type (io->ncid, io->xtype, NULL, NULL, &basetype, NULL, NULL));
 
   for (ii=0; ii<cnt; ii++) {
-    R_nc_c2r_init (&tmpio, vbuf[ii].p, io->ncid, basetype, -1, &(vbuf[ii].len),
-                   io->rawchar, io->fitnum, 0, NULL, NULL, NULL, NULL, NULL);
-    SET_VECTOR_ELT (io->rxp, ii, R_nc_c2r (&tmpio));
+    tmprxp = PROTECT(R_nc_c2r_init (&tmpio, &(vbuf[ii].p), io->ncid, basetype, -1,
+                       &(vbuf[ii].len), io->rawchar, io->fitnum,
+                       0, NULL, NULL, NULL, NULL, NULL));
+    R_nc_c2r (&tmpio);
+    SET_VECTOR_ELT (io->rxp, ii, tmprxp);
     nc_free_vlen(&(vbuf[ii]));
+    UNPROTECT(1);
   }
 }
 
@@ -810,14 +834,14 @@ R_nc_raw_opaque (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim)
   size_t cnt, size;
   R_nc_check (nc_inq_user_type (ncid, xtype, NULL, &size, NULL, NULL, NULL));
   cnt = R_nc_length (ndim, xdim);
-  if (xlength (rv) < (cnt * size)) {
-    RERROR (RNC_EDATALEN);
+  if ((size_t) xlength (rv) < (cnt * size)) {
+    error (RNC_EDATALEN);
   }
   return (const char *) RAW (rv);
 }
 
 
-static void
+static SEXP
 R_nc_opaque_raw_init (R_nc_buf *io)
 {
   int ndim;
@@ -840,11 +864,13 @@ R_nc_opaque_raw_init (R_nc_buf *io)
   }
   xdim[ndim] = size;
 
-  io->rxp = R_nc_allocArray (RAWSXP, ndim + 1, xdim);
+  io->rxp = PROTECT(R_nc_allocArray (RAWSXP, ndim + 1, xdim));
   io->rbuf = RAW (io->rxp);
   if (!io->cbuf) {
     io->cbuf = io->rbuf;
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -868,7 +894,7 @@ static void *
 R_nc_factor_enum (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim)
 {
   SEXP levels;
-  size_t size, imem, nmem, ilev, nlev, *ilev2mem, ifac, nfac;
+  size_t size, imem, nmem, ilev, nlev, *ilev2mem, ifac, nfac, cnt;
   char *memnames, *memname, *memvals, *memval, *out;
   const char **levnames;
   int ismatch, *in, inval;
@@ -878,7 +904,7 @@ R_nc_factor_enum (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim
 
   levels = getAttrib (rv, R_LevelsSymbol);
   if (!isString (levels)) {
-    RERROR ("Expected character vector for levels of factor array")
+    error ("Expected character vector for levels of factor array");
   }
 
   nlev = xlength (levels);
@@ -914,21 +940,25 @@ R_nc_factor_enum (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim
       }
     }
     if (!ismatch) {
-      RERROR ("Level has no matching member in enum type")
+      error ("Level has no matching member in enum type");
     }
   }
 
   /* Convert factor indices to enum values */
   nfac = xlength (rv);
+  cnt = R_nc_length (ndim, xdim);
+  if (nfac < cnt) {
+    error (RNC_EDATALEN);
+  }
   out = R_alloc (nfac, size);
 
   for (ifac=0; ifac<nfac; ifac++) {
     inval = in[ifac];
-    if (0 < inval && inval <= nlev) {
+    if (0 < inval && (size_t) inval <= nlev) {
       imem = ilev2mem[inval-1];
       memcpy(out + ifac*size, memvals + imem*size, size);
     } else {
-      RERROR ("Invalid index in factor")
+      error ("Invalid index in factor");
     }
   }
 
@@ -936,16 +966,18 @@ R_nc_factor_enum (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim
 }
 
 
-static void
+static SEXP
 R_nc_enum_factor_init (R_nc_buf *io)
 {
   size_t size;
-  io->rxp = R_nc_allocArray (INTSXP, io->ndim, io->xdim);
+  io->rxp = PROTECT(R_nc_allocArray (INTSXP, io->ndim, io->xdim));
   io->rbuf = INTEGER (io->rxp);
   if (!io->cbuf) {
     R_nc_check (nc_inq_type (io->ncid, io->xtype, NULL, &size));
     io->cbuf = R_alloc (xlength (io->rxp), size);
   }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -972,27 +1004,32 @@ R_nc_char_symbol (char *in, size_t size, char *work)
 static void
 R_nc_enum_factor (R_nc_buf *io)
 {
-  SEXP levels, classname, env, cmd, symbol, value;
+  SEXP levels, env, cmd, symbol, index;
   size_t size, nmem, ifac, nfac;
   char *memname, *memval, *work, *inval;
   int ncid, imem, imemmax, *out;
   nc_type xtype;
 
-  /* Read values and names of netcdf enum members.
-     Store names in an R character vector for use as R factor levels.
-     Store values and their R indices (1-based) in a hashed environment.
-     The env is PROTECTed, so individual variables need not be.
-     But values do need PROTECTing before assignment to env, 
-     otherwise gctorture reveals problems.
-     I'm not sure if symbols need PROTECTing, but better safe than sorry.
-   */
+  /* Get size and number of enum members */
   ncid = io->ncid;
   xtype = io->xtype;
   R_nc_check (nc_inq_enum(ncid, xtype, NULL, NULL, &size, &nmem));
-  cmd = R_nc_protect (lang1 (install ("new.env")));
-  env = R_nc_protect (eval (cmd, R_BaseEnv));
 
-  levels = R_nc_allocArray (STRSXP, -1, &nmem);
+  /* Set attributes for R factor */
+  levels = PROTECT(R_nc_allocArray (STRSXP, -1, &nmem));
+  setAttrib(io->rxp, R_LevelsSymbol, levels);
+  setAttrib(io->rxp, R_ClassSymbol, mkString("factor"));
+
+  /* Create a hashed environment for value-index pairs.
+     Members inherit PROTECTion from the env.
+   */
+  cmd = PROTECT(lang1 (install ("new.env")));
+  env = PROTECT(eval (cmd, R_BaseEnv));
+
+  /* Read values and names of netcdf enum members.
+     Store names as R factor levels.
+     Store values and their R indices (1-based) in hashed environment.
+   */
   memname = R_alloc (nmem, NC_MAX_NAME+1);
   memval = R_alloc (1, size);
   work = R_alloc (2*size+2, 1);
@@ -1002,8 +1039,8 @@ R_nc_enum_factor (R_nc_buf *io)
     R_nc_check (nc_inq_enum_member (ncid, xtype, imem, memname, memval));
     SET_STRING_ELT (levels, imem, mkChar (memname));
     symbol = PROTECT (R_nc_char_symbol (memval, size, work));
-    value = PROTECT (ScalarInteger (imem+1));
-    defineVar (symbol, value, env);
+    index = PROTECT (ScalarInteger (imem+1));
+    defineVar (symbol, index, env);
     UNPROTECT(2);
   }
 
@@ -1014,21 +1051,18 @@ R_nc_enum_factor (R_nc_buf *io)
 
   out = io->rbuf;
   for (ifac=0, inval=io->cbuf; ifac<nfac; ifac++, inval+=size) {
-    symbol = PROTECT (R_nc_char_symbol (inval, size, work));
-    value = findVarInFrame3 (env, symbol, TRUE);
+    symbol = PROTECT(R_nc_char_symbol (inval, size, work));
+    index = findVarInFrame3 (env, symbol, TRUE);
     UNPROTECT(1);
-    if (value == R_UnboundValue) {
-      R_nc_error ("Unknown enum value in variable");
+    if (index == R_UnboundValue) {
+      error ("Unknown enum value in variable");
     } else {
-      out[ifac] = INTEGER (value)[0];
+      out[ifac] = INTEGER (index)[0];
     }
   }
 
-  /* Set attributes for R factor */
-  setAttrib(io->rxp, R_LevelsSymbol, levels);
-  classname = R_nc_protect (allocVector (STRSXP, 1));
-  SET_STRING_ELT(classname, 0, mkChar("factor"));
-  setAttrib(io->rxp, R_ClassSymbol, classname);
+  /* Allow garbage collection of env and levels */
+  UNPROTECT(3);
 }
 
 
@@ -1053,13 +1087,13 @@ R_nc_vecsxp_compound (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *
   R_nc_check (nc_inq_compound(ncid, xtype, NULL, &size, &nfld));
 
   /* Check names attribute of R list */
-  namelist = R_nc_protect (getAttrib (rv, R_NamesSymbol));
+  namelist = PROTECT(getAttrib (rv, R_NamesSymbol));
   if (!isString (namelist)) {
-    R_nc_error ("Named list required for conversion to compound type");
+    error ("Named list required for conversion to compound type");
   }
   nlist = xlength (namelist);
   if (nlist < nfld) {
-    R_nc_error ("Not enough fields in list for conversion to compound type");
+    error ("Not enough fields in list for conversion to compound type");
   }
 
   /* Allocate memory for compound array,
@@ -1095,7 +1129,7 @@ R_nc_vecsxp_compound (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *
       }
     }
     if (!ismatch) {
-      R_nc_error ("Name of compound field not found in input list");
+      error ("Name of compound field not found in input list");
     }
 
     /* Convert the field from R to C.
@@ -1121,11 +1155,12 @@ R_nc_vecsxp_compound (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *
     vmaxset (highwater);
   }
 
+  UNPROTECT(1);
   return bufout;
 }
 
 
-static void
+static SEXP
 R_nc_compound_vecsxp_init (R_nc_buf *io)
 {
   size_t size, nfld, cnt;
@@ -1139,20 +1174,23 @@ R_nc_compound_vecsxp_init (R_nc_buf *io)
    */
   if (R_nc_redef (io->ncid) == NC_NOERR) {
     /* Dataset must be writable because it is now in define mode */
-    R_nc_error ("Please read compound type from a read-only dataset");
+    error ("Please read compound type from a read-only dataset");
   }
 
   /* Get number of fields in compound type */
   R_nc_check (nc_inq_compound(io->ncid, io->xtype, NULL, &size, &nfld));
 
   /* Allocate memory for output list */
-  io->rxp = R_nc_allocArray (VECSXP, -1, &nfld);
+  io->rxp = PROTECT(R_nc_allocArray (VECSXP, -1, &nfld));
 
   /* Allocate memory for compound array */
   if (!io->cbuf) {
     cnt = R_nc_length (io->ndim, io->xdim);
     io->cbuf = R_alloc (cnt, size);
   }
+
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
@@ -1177,7 +1215,7 @@ R_nc_compound_vecsxp (R_nc_buf *io)
   cnt = R_nc_length (io->ndim, io->xdim);
 
   /* Set names attribute of R list */
-  namelist = R_nc_allocArray (STRSXP, -1, &nfld);
+  namelist = PROTECT(R_nc_allocArray (STRSXP, -1, &nfld));
   setAttrib(io->rxp, R_NamesSymbol, namelist);
 
   /* Convert each field in turn */
@@ -1221,8 +1259,10 @@ R_nc_compound_vecsxp (R_nc_buf *io)
     fldcnt = R_nc_length (ndimfld, dimslice+ndim);
 
     /* Prepare to convert field data from C to R */
-    buffld = R_nc_c2r_init (&iofld, NULL, ncid, typefld, ndimslice, dimslice,
-               io->rawchar, io->fitnum, 0, NULL, NULL, NULL, NULL, NULL);
+    buffld = NULL;
+    rxpfld = PROTECT(R_nc_c2r_init (&iofld, (void **) &buffld, ncid, typefld,
+               ndimslice, dimslice, io->rawchar, io->fitnum,
+               0, NULL, NULL, NULL, NULL, NULL));
 
     /* Copy elements from the compound array into the field array */
     fldlen = fldsize * fldcnt;
@@ -1231,14 +1271,16 @@ R_nc_compound_vecsxp (R_nc_buf *io)
     }
 
     /* Convert field data from C to R */
-    rxpfld = R_nc_c2r (&iofld);
+    R_nc_c2r (&iofld);
 
     /* Insert field data into R list */
     SET_VECTOR_ELT (io->rxp, ifld, rxpfld);
 
     /* Allow memory from R_alloc since vmaxget to be reclaimed */
+    UNPROTECT(1);
     vmaxset (highwater);
   }
+  UNPROTECT(1);
 }
 
 
@@ -1362,11 +1404,11 @@ R_nc_r2c (SEXP rv, int ncid, nc_type xtype, int ndim, const size_t *xdim,
     }
     break;
   }
-  RERROR (RNC_EDATATYPE);
+  error (RNC_EDATATYPE);
 }
 
-void * \
-R_nc_c2r_init (R_nc_buf *io, void *cbuf,
+SEXP \
+R_nc_c2r_init (R_nc_buf *io, void **cbuf,
                int ncid, nc_type xtype, int ndim, const size_t *xdim,
                int rawchar, int fitnum, size_t fillsize,
                const void *fill, const void *min, const void *max,
@@ -1375,12 +1417,12 @@ R_nc_c2r_init (R_nc_buf *io, void *cbuf,
   int class;
 
   if (!io) {
-    RERROR ("Pointer to R_nc_buf must not be NULL in R_nc_c2r_init");
+    error ("Pointer to R_nc_buf must not be NULL in R_nc_c2r_init");
   }
 
   /* Initialise the R_nc_buf, making copies of pointer arguments */
   io->rxp = NULL;
-  io->cbuf = cbuf;
+  io->cbuf = NULL;
   io->rbuf = NULL;
   io->xtype = xtype;
   io->ncid = ncid;
@@ -1395,6 +1437,10 @@ R_nc_c2r_init (R_nc_buf *io, void *cbuf,
   io->scale = NULL;
   io->add = NULL;
 
+  if (cbuf) {
+    io->cbuf = *cbuf;
+  }
+
   if (xdim) {
     if (ndim > 0) {
       io->xdim = (size_t *) R_alloc (ndim, sizeof(size_t));
@@ -1440,59 +1486,64 @@ R_nc_c2r_init (R_nc_buf *io, void *cbuf,
     case NC_USHORT:
     case NC_INT:
       if (fitnum && !scale && !add) {
-        R_nc_c2r_int_init (io);
+        PROTECT(R_nc_c2r_int_init (io));
         break;
       }
     case NC_INT64:
     case NC_UINT64:
       if (fitnum && !scale && !add) {
-        R_nc_c2r_bit64_init (io);
+        PROTECT(R_nc_c2r_bit64_init (io));
         classgets(io->rxp, mkString("integer64"));
         break;
       }
     case NC_UINT:
     case NC_FLOAT:
     case NC_DOUBLE:
-      R_nc_c2r_dbl_init (io);
+      PROTECT(R_nc_c2r_dbl_init (io));
       break;
     case NC_CHAR:
       if (rawchar) {
-        R_nc_char_raw_init (io);
+        PROTECT(R_nc_char_raw_init (io));
       } else {
-        R_nc_char_strsxp_init (io);
+        PROTECT(R_nc_char_strsxp_init (io));
       }
       break;
     case NC_STRING:
-      R_nc_str_strsxp_init (io);
+      PROTECT(R_nc_str_strsxp_init (io));
       break;
     default:
       if (xtype > NC_MAX_ATOMIC_TYPE) {
         R_nc_check (nc_inq_user_type (ncid, xtype, NULL, NULL, NULL, NULL, &class));
         switch (class) {
         case NC_COMPOUND:
-          R_nc_compound_vecsxp_init (io);
+          PROTECT(R_nc_compound_vecsxp_init (io));
           break;
         case NC_ENUM:
-          R_nc_enum_factor_init (io);
+          PROTECT(R_nc_enum_factor_init (io));
           break;
         case NC_VLEN:
-          R_nc_vlen_vecsxp_init (io);
+          PROTECT(R_nc_vlen_vecsxp_init (io));
           break;
         case NC_OPAQUE:
-          R_nc_opaque_raw_init (io);
+          PROTECT(R_nc_opaque_raw_init (io));
           break;
         default:
-          RERROR (RNC_ETYPEDROP);
+          error (RNC_ETYPEDROP);
         }
       } else {
-        RERROR (RNC_ETYPEDROP);
+        error (RNC_ETYPEDROP);
       }
   }
-  return io->cbuf;
+
+  if (cbuf) {
+    *cbuf = io->cbuf;
+  }
+  UNPROTECT(1);
+  return io->rxp;
 }
 
 
-SEXP
+void
 R_nc_c2r (R_nc_buf *io)
 {
   int unpack, class;
@@ -1613,13 +1664,12 @@ R_nc_c2r (R_nc_buf *io)
           R_nc_opaque_raw (io);
           break;
         default:
-          RERROR (RNC_ETYPEDROP);
+          error (RNC_ETYPEDROP);
         }
       } else {
-        RERROR (RNC_ETYPEDROP);
+        error (RNC_ETYPEDROP);
       }
   }
-  return io->rxp;
 }
 
 
@@ -1678,7 +1728,7 @@ FUN (SEXP rv, size_t N, TYPE fillval) \
   } else if (isInteger (rv)) { \
     voidbuf = R_nc_r2c_int_##TYPENAME (rv, 1, &nr, sizeof(TYPE), &fillval, NULL, NULL); \
   } else { \
-    RERROR ("Unsupported R type in R_NC_DIM_R2C"); \
+    error ("Unsupported R type in R_NC_DIM_R2C"); \
   } \
   memcpy (cv, voidbuf, nr*sizeof (TYPE)); \
 \
