@@ -2,7 +2,7 @@
  *
  *  Name:       variable.c
  *
- *  Version:    2.0-3
+ *  Version:    2.3-1
  *
  *  Purpose:    NetCDF variable functions for RNetCDF
  *
@@ -59,12 +59,17 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
               SEXP filter_params)
 {
   int ncid, ii, jj, *dimids, ndims, varid, chunkmode, format, withnc4;
-  int deflate_mode, deflate_level, shuffle_mode, endian_mode, fletcher_mode;
-  int filter_mode, filtid, *filtparm;
+  int deflate_mode, deflate_level, shuffle_mode, fletcher_mode;
   size_t *chunksize_t;
   nc_type xtype;
   const char *varnamep;
-  SEXP result;
+
+#ifdef HAVE_NC_INQ_VAR_ENDIAN
+  int endian_mode;
+#endif
+#ifdef HAVE_NC_INQ_VAR_FILTER
+  int filter_mode, filtid, *filtparm;
+#endif
 
   /*-- Convert arguments to netcdf ids ----------------------------------------*/
   ncid = asInteger (nc);
@@ -87,7 +92,7 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
   if (withnc4) {
     chunkmode = asLogical (chunking);
     if (ndims == 0) {
-      /* Chunking is not relevant to scalar variables */
+      /* Chunking is not allowed for scalar variables */
       chunkmode = NA_LOGICAL;
     }
     if (chunkmode == TRUE) {
@@ -119,9 +124,11 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
 
     fletcher_mode = (asLogical (fletcher32) == TRUE);
 
+#ifdef HAVE_NC_INQ_VAR_FILTER
     filtid = asInteger (filter_id);
     filter_mode = (filtid != NA_INTEGER);
     filtparm = INTEGER (filter_params);
+#endif
   }
 
   /*-- Enter define mode ------------------------------------------------------*/
@@ -159,11 +166,9 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
         xlength (filter_params), (const unsigned int*) filtparm));
     }
 #endif
-
   }
 
-  result = R_nc_protect (ScalarInteger (varid));
-  RRETURN(result);
+  return ScalarInteger (varid);
 }
 
 
@@ -194,6 +199,7 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
 /* Find attributes related to missing values for a netcdf variable.
    On exit, relevant parameters are returned via double pointers to
      fill, min and max, which are either NULL or allocated by R_alloc.
+     The function returns the in-memory size (bytes) of a missing value.
    Argument mode specifies the attributes used for missing values:
      0 - _FillValue, or missing_value
      1 - _FillValue only
@@ -203,31 +209,26 @@ R_nc_def_var (SEXP nc, SEXP varname, SEXP type, SEXP dims,
          http://www.unidata.ucar.edu/software/netcdf/docs/attribute_conventions.html
    Example: R_nc_miss_att (ncid, varid, mode, &fill, &min, &max);
   */
-static void
+static size_t
 R_nc_miss_att (int ncid, int varid, int mode,
                void **fill, void **min, void **max)
 {
   size_t cnt, size;
-  int class;
-  nc_type atype, xtype, basetype;
+  nc_type atype, xtype;
   char *range;
   *fill = NULL;
   *min = NULL;
   *max = NULL;
 
-  /* Get details about type of netcdf variable */
+  /* Get details about type and size of netcdf variable */
   R_nc_check (nc_inq_vartype (ncid, varid, &xtype));
-  if (xtype > NC_MAX_ATOMIC_TYPE) {
-    /* Use base type of vlen or enum type */
-    R_nc_check (nc_inq_user_type (ncid, xtype, NULL, NULL, &basetype, NULL, &class));
-    if (class == NC_ENUM || class == NC_VLEN) {
-      xtype = basetype;
-    } else {
-      /* Other user-defined types can be handled by users,
-         based on any convention they choose.
-       */
-      return;
-    }
+  if (xtype == NC_CHAR ||
+      xtype == NC_STRING ||
+      xtype > NC_MAX_ATOMIC_TYPE) {
+    /* NetCDF attribute conventions describe the handling of missing values
+       in atomic numeric types. Let users handle other types as needed.
+     */
+    return 0;
   }
   R_nc_check (nc_inq_type (ncid, xtype, NULL, &size));
 
@@ -247,7 +248,7 @@ R_nc_miss_att (int ncid, int varid, int mode,
 
   } else if (mode == 3) {
     /* Let user code handle missing values */
-    return;
+    return 0;
 
   } else if (mode == 4) {
 
@@ -372,7 +373,7 @@ R_nc_miss_att (int ncid, int varid, int mode,
             **(unsigned long long **) fill = NC_FILL_UINT64;
             break;
           default:
-            return;
+            error ("Default fill value not implemented");
         }
       }
 
@@ -406,15 +407,16 @@ R_nc_miss_att (int ncid, int varid, int mode,
             FILL2RANGE_REAL(double, DBL_EPSILON);
             break;
           default:
-            return;
+            error ("Default valid range not implemented");
         }
       }
 
     }
   } else {
-    R_nc_error ("Unknown mode for handling missing values");
+    error ("Unknown mode for handling missing values");
 
   }
+  return size;
 }
 
 
@@ -457,9 +459,14 @@ R_nc_get_var (SEXP nc, SEXP var, SEXP start, SEXP count,
   R_nc_buf io;
   double add, scale, *addp=NULL, *scalep=NULL;
   void *fillp=NULL, *minp=NULL, *maxp=NULL;
+  size_t fillsize;
+
+#ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
+  int storeprop;
   size_t bytes, slots;
   float preemption;
   double bytes_in, slots_in, preempt_in;
+#endif
 
   /*-- Convert arguments ------------------------------------------------------*/
   ncid = asInteger (nc);
@@ -473,8 +480,10 @@ R_nc_get_var (SEXP nc, SEXP var, SEXP start, SEXP count,
 
   /*-- Chunk cache options for netcdf4 files ----------------------------------*/
 #ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
-  if (nc_get_var_chunk_cache(ncid, varid,
-                             &bytes, &slots, &preemption) == NC_NOERR) {
+  R_nc_check (nc_inq_var_chunking (ncid, varid, &storeprop, NULL));
+  if (storeprop == NC_CHUNKED) {
+    R_nc_check (nc_get_var_chunk_cache(ncid, varid,
+                                       &bytes, &slots, &preemption));
     bytes_in = asReal (cache_bytes);
     slots_in = asReal (cache_slots);
     preempt_in = asReal (cache_preemption);
@@ -507,7 +516,7 @@ R_nc_get_var (SEXP nc, SEXP var, SEXP start, SEXP count,
   }
 
   /*-- Get fill attributes (if any) -------------------------------------------*/
-  R_nc_miss_att (ncid, varid, inamode, &fillp, &minp, &maxp);
+  fillsize = R_nc_miss_att (ncid, varid, inamode, &fillp, &minp, &maxp);
 
   /*-- Get packing attributes (if any) ----------------------------------------*/
   if (isunpack) {
@@ -520,15 +529,17 @@ R_nc_get_var (SEXP nc, SEXP var, SEXP start, SEXP count,
   R_nc_check (R_nc_enddef (ncid));
 
   /*-- Allocate memory and read variable from file ----------------------------*/
-  buf = R_nc_c2r_init (&io, NULL, ncid, xtype, ndims, ccount,
-                       israw, isfit, fillp, minp, maxp, scalep, addp);
+  buf = NULL;
+  result = PROTECT(R_nc_c2r_init (&io, &buf, ncid, xtype, ndims, ccount,
+                     israw, isfit, fillsize, fillp, minp, maxp, scalep, addp));
 
   if (R_nc_length (ndims, ccount) > 0) {
     R_nc_check (nc_get_vara (ncid, varid, cstart, ccount, buf));
   }
-  result = R_nc_c2r (&io);
+  R_nc_c2r (&io);
 
-  RRETURN (result);
+  UNPROTECT(1);
+  return result;
 }
 
 
@@ -540,18 +551,29 @@ SEXP
 R_nc_inq_var (SEXP nc, SEXP var)
 {
   int ncid, varid, idim, ndims, natts, *dimids, storeprop, format, withnc4;
-  int shuffle, deflate, deflate_level, endian, fletcher;
-  int status, szip_options, szip_bits;
-  int filter_id;
-  size_t filter_nparams;
-  size_t *chunksize_t, cache_bytes, cache_slots;
-  float cache_preemption;
+  int shuffle, deflate, deflate_level, fletcher;
+  int status;
+  size_t *chunksize_t;
   double *chunkdbl;
   char varname[NC_MAX_NAME + 1], vartype[NC_MAX_NAME+1];
   nc_type xtype;
-  SEXP result, rdimids, rchunks, rbytes, rslots, rpreempt,
-       rshuffle, rdeflate, rendian, rfletcher,
-       rszip_options, rszip_bits, rfilter_id, rfilter_params;
+  SEXP result, rdimids, rchunks;
+
+#ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
+  size_t cache_bytes, cache_slots;
+  float cache_preemption;
+#endif
+#ifdef HAVE_NC_INQ_VAR_ENDIAN
+  int endian;
+#endif
+#ifdef HAVE_NC_INQ_VAR_SZIP
+  int szip_options, szip_bits;
+#endif
+#ifdef HAVE_NC_INQ_VAR_FILTER
+  int filter_id;
+  size_t filter_nparams;
+  SEXP rfilter_params;
+#endif
 
   /*-- Convert arguments to netcdf ids ----------------------------------------*/
   ncid = asInteger (nc);
@@ -564,169 +586,160 @@ R_nc_inq_var (SEXP nc, SEXP var)
 
   R_nc_check (nc_inq_var (ncid, varid, varname, &xtype, &ndims, NULL, &natts));
 
-  if (ndims > 0) {
-    rdimids = R_nc_protect (allocVector (INTSXP, ndims));
-    dimids = INTEGER (rdimids);
-    R_nc_check (nc_inq_vardimid (ncid, varid, dimids));
-    /* Return dimension ids in reverse (Fortran) order */
-    R_nc_rev_int (dimids, ndims);
-
-    rchunks = R_NilValue;
-    if (withnc4) {
-      R_nc_check (nc_inq_var_chunking (ncid, varid, &storeprop, NULL));
-      if (storeprop == NC_CHUNKED) {
-	rchunks = R_nc_protect (allocVector (REALSXP, ndims));
-	chunkdbl = REAL (rchunks);
-	chunksize_t = (size_t *) R_alloc (ndims, sizeof(size_t));
-	R_nc_check (nc_inq_var_chunking (ncid, varid, NULL, chunksize_t));
-	/* Return chunk sizes as double precision in reverse (Fortran) order */
-	R_nc_rev_size (chunksize_t, ndims);
-	for (idim=0; idim<ndims; idim++) {
-	  chunkdbl[idim] = chunksize_t[idim];
-	}
-      }
-
-#ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
-      R_nc_check (nc_get_var_chunk_cache (ncid, varid, &cache_bytes,
-                                          &cache_slots, &cache_preemption));
-      rbytes = R_nc_protect (ScalarReal (cache_bytes));
-      rslots = R_nc_protect (ScalarReal (cache_slots));
-      rpreempt = R_nc_protect (ScalarReal (cache_preemption));
-#else
-      rbytes = R_NilValue;
-      rslots = R_NilValue;
-      rpreempt = R_NilValue;
-#endif
-    }
-
-  } else {
-    /* Return single NA for scalars */
-    rdimids = R_nc_protect (ScalarInteger (NA_INTEGER));
-
-    /* Chunks not defined for scalars */
-    rchunks = R_NilValue;
-    rbytes = R_nc_protect (ScalarReal (NA_REAL));
-    rslots = R_nc_protect (ScalarReal (NA_REAL));
-    rpreempt = R_nc_protect (ScalarReal (NA_REAL));
-  }
-
-  if (withnc4) {
-    /* deflate and shuffle */
-    R_nc_check (nc_inq_var_deflate (ncid, varid, &shuffle,
-                                    &deflate, &deflate_level));
-    if (deflate) {
-      rdeflate = R_nc_protect (ScalarInteger (deflate_level));
-    } else {
-      rdeflate = R_nc_protect (ScalarInteger (NA_INTEGER));
-    }
-    rshuffle = R_nc_protect (ScalarLogical (shuffle));
-
-    /* endian */
-#ifdef HAVE_NC_INQ_VAR_ENDIAN
-    R_nc_check (nc_inq_var_endian (ncid, varid, &endian));
-    if (endian == NC_ENDIAN_LITTLE) {
-      rendian = R_nc_protect (ScalarLogical (0));
-    } else if (endian == NC_ENDIAN_BIG) {
-      rendian = R_nc_protect (ScalarLogical (1));
-    } else {
-      rendian = R_nc_protect (ScalarLogical (NA_LOGICAL));
-    }
-#else
-    rendian = R_NilValue;
-#endif
-
-    /* fletcher32 */
-    R_nc_check (nc_inq_var_fletcher32 (ncid, varid, &fletcher));
-    rfletcher = R_nc_protect (ScalarLogical (fletcher == NC_FLETCHER32));
-
-    /* szip */
-#ifdef HAVE_NC_INQ_VAR_SZIP
-    status = nc_inq_var_szip (ncid, varid, &szip_options, &szip_bits);
-    if (status == NC_NOERR) {
-      rszip_options = R_nc_protect (ScalarInteger (szip_options));
-      rszip_bits = R_nc_protect (ScalarInteger (szip_bits));
-#  if defined NC_EFILTER
-    } else if (status == NC_EFILTER) {
-      rszip_options = R_nc_protect (ScalarInteger (NA_INTEGER));
-      rszip_bits = R_nc_protect (ScalarInteger (NA_INTEGER));
-#  endif
-    } else {
-      R_nc_check (status);
-      return R_NilValue;
-    }
-#else
-    rszip_options = R_NilValue;
-    rszip_bits = R_NilValue;
-#endif
-
-    /* filter */
-#ifdef HAVE_NC_INQ_VAR_FILTER
-    status = nc_inq_var_filter (ncid, varid,
-                                (unsigned int *) &filter_id,
-                                &filter_nparams, NULL);
-    if (status == NC_NOERR) {
-      rfilter_id = R_nc_protect (ScalarInteger (filter_id));
-      rfilter_params = R_nc_protect (allocVector (INTSXP, filter_nparams));
-      if (filter_nparams > 0) {
-        R_nc_check (nc_inq_var_filter (ncid, varid, NULL, NULL,
-                      (unsigned int *) INTEGER (rfilter_params)));
-      }
-    } else if (status == NC_EFILTER) {
-      rfilter_id = R_nc_protect (ScalarInteger (NA_INTEGER));
-      rfilter_params = R_nc_protect (ScalarInteger (NA_INTEGER));
-    } else {
-      R_nc_check (status);
-      return R_NilValue;
-    }
-#else
-    rfilter_id = R_NilValue;
-    rfilter_params = R_NilValue;
-#endif
-
-  } else {
-    rdeflate = R_NilValue;
-    rshuffle = R_NilValue;
-    rendian = R_NilValue;
-    rfletcher = R_NilValue;
-    rszip_bits = R_NilValue;
-    rszip_options = R_NilValue;
-    rfilter_id = R_NilValue;
-    rfilter_params = R_NilValue;
-  }
-
-  /*-- Convert nc_type to char ------------------------------------------------*/
   R_nc_check (R_nc_type2str (ncid, xtype, vartype));
 
-  /*-- Construct the output list ----------------------------------------------*/
   if (withnc4) {
-    result = R_nc_protect (allocVector (VECSXP, 18));
+    result = PROTECT(allocVector (VECSXP, 18));
   } else {
-    result = R_nc_protect (allocVector (VECSXP, 6));
+    result = PROTECT(allocVector (VECSXP, 6));
   }
 
   SET_VECTOR_ELT (result, 0, ScalarInteger (varid));
   SET_VECTOR_ELT (result, 1, mkString (varname));
   SET_VECTOR_ELT (result, 2, mkString (vartype));
   SET_VECTOR_ELT (result, 3, ScalarInteger (ndims));
-  SET_VECTOR_ELT (result, 4, rdimids);
   SET_VECTOR_ELT (result, 5, ScalarInteger (natts));
 
-  if (withnc4) {
-    SET_VECTOR_ELT (result, 6, rchunks);
-    SET_VECTOR_ELT (result, 7, rbytes);
-    SET_VECTOR_ELT (result, 8, rslots);
-    SET_VECTOR_ELT (result, 9, rpreempt);
-    SET_VECTOR_ELT (result, 10, rdeflate);
-    SET_VECTOR_ELT (result, 11, rshuffle);
-    SET_VECTOR_ELT (result, 12, rendian);
-    SET_VECTOR_ELT (result, 13, rfletcher);
-    SET_VECTOR_ELT (result, 14, rszip_options);
-    SET_VECTOR_ELT (result, 15, rszip_bits);
-    SET_VECTOR_ELT (result, 16, rfilter_id);
-    SET_VECTOR_ELT (result, 17, rfilter_params);
+  if (ndims > 0) {
+    rdimids = PROTECT(allocVector (INTSXP, ndims));
+    SET_VECTOR_ELT (result, 4, rdimids);
+    UNPROTECT(1);
+    dimids = INTEGER (rdimids);
+    R_nc_check (nc_inq_vardimid (ncid, varid, dimids));
+    /* Return dimension ids in reverse (Fortran) order */
+    R_nc_rev_int (dimids, ndims);
+  } else {
+    /* Return single NA for scalar dimensions */
+    SET_VECTOR_ELT (result, 4, ScalarInteger (NA_INTEGER));
   }
 
-  RRETURN(result);
+  if (withnc4) {
+    R_nc_check (nc_inq_var_chunking (ncid, varid, &storeprop, NULL));
+
+    if (storeprop == NC_CHUNKED) {
+      rchunks = PROTECT(allocVector (REALSXP, ndims));
+      SET_VECTOR_ELT (result, 6, rchunks);
+      UNPROTECT(1);
+      chunkdbl = REAL (rchunks);
+      chunksize_t = (size_t *) R_alloc (ndims, sizeof(size_t));
+      R_nc_check (nc_inq_var_chunking (ncid, varid, NULL, chunksize_t));
+      /* Return chunk sizes as double precision in reverse (Fortran) order */
+      R_nc_rev_size (chunksize_t, ndims);
+      for (idim=0; idim<ndims; idim++) {
+	chunkdbl[idim] = chunksize_t[idim];
+      }
+
+#ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
+      R_nc_check (nc_get_var_chunk_cache (ncid, varid, &cache_bytes,
+					  &cache_slots, &cache_preemption));
+      SET_VECTOR_ELT (result, 7, ScalarReal (cache_bytes));
+      SET_VECTOR_ELT (result, 8, ScalarReal (cache_slots));
+      SET_VECTOR_ELT (result, 9, ScalarReal (cache_preemption));
+#else
+      SET_VECTOR_ELT (result, 7, R_NilValue);
+      SET_VECTOR_ELT (result, 8, R_NilValue);
+      SET_VECTOR_ELT (result, 9, R_NilValue);
+#endif
+    } else {
+      /* Chunks not defined */
+      SET_VECTOR_ELT (result, 6, R_NilValue);
+      SET_VECTOR_ELT (result, 7, ScalarReal (NA_REAL));
+      SET_VECTOR_ELT (result, 8, ScalarReal (NA_REAL));
+      SET_VECTOR_ELT (result, 9, ScalarReal (NA_REAL));
+    }
+
+    /* deflate and shuffle */
+    R_nc_check (nc_inq_var_deflate (ncid, varid, &shuffle,
+                                    &deflate, &deflate_level));
+    if (deflate) {
+      SET_VECTOR_ELT (result, 10, ScalarInteger (deflate_level));
+    } else {
+      SET_VECTOR_ELT (result, 10, ScalarInteger (NA_INTEGER));
+    }
+    SET_VECTOR_ELT (result, 11, ScalarLogical (shuffle));
+
+    /* endian */
+#ifdef HAVE_NC_INQ_VAR_ENDIAN
+    R_nc_check (nc_inq_var_endian (ncid, varid, &endian));
+    if (endian == NC_ENDIAN_LITTLE) {
+      SET_VECTOR_ELT (result, 12, ScalarLogical(0));
+    } else if (endian == NC_ENDIAN_BIG) {
+      SET_VECTOR_ELT (result, 12, ScalarLogical(1));
+    } else {
+      SET_VECTOR_ELT (result, 12, ScalarLogical(NA_LOGICAL));
+    }
+#else
+    SET_VECTOR_ELT (result, 12, R_NilValue);
+#endif
+
+    /* fletcher32 */
+    R_nc_check (nc_inq_var_fletcher32 (ncid, varid, &fletcher));
+    SET_VECTOR_ELT (result, 13, ScalarLogical (fletcher == NC_FLETCHER32));
+
+    /* szip */
+#ifdef HAVE_NC_INQ_VAR_SZIP
+    status = nc_inq_var_szip (ncid, varid, &szip_options, &szip_bits);
+    if (status == NC_NOERR) {
+      if (szip_options == 0) {
+        /* netcdf>=4.7.4 sets results to 0 if szip is not used */
+        SET_VECTOR_ELT (result, 14, ScalarInteger (NA_INTEGER));
+        SET_VECTOR_ELT (result, 15, ScalarInteger (NA_INTEGER));
+      } else {
+        SET_VECTOR_ELT (result, 14, ScalarInteger (szip_options));
+        SET_VECTOR_ELT (result, 15, ScalarInteger (szip_bits));
+      }
+#  if defined NC_EFILTER
+    } else if (status == NC_EFILTER) {
+      /* netcdf<4.7.4 returns NC_EFILTER if szip is not used */
+      SET_VECTOR_ELT (result, 14, ScalarInteger (NA_INTEGER));
+      SET_VECTOR_ELT (result, 15, ScalarInteger (NA_INTEGER));
+#  endif
+    } else {
+      error (nc_strerror (status));
+    }
+#else
+    SET_VECTOR_ELT (result, 14, R_NilValue);
+    SET_VECTOR_ELT (result, 15, R_NilValue);
+#endif
+
+    /* filter */
+#ifdef HAVE_NC_INQ_VAR_FILTER
+    if (storeprop == NC_CHUNKED) {
+      status = nc_inq_var_filter (ncid, varid,
+                                  (unsigned int *) &filter_id,
+                                  &filter_nparams, NULL);
+      if (status == NC_NOERR) {
+	SET_VECTOR_ELT (result, 16, ScalarInteger (filter_id));
+	rfilter_params = PROTECT(allocVector (INTSXP, filter_nparams));
+	SET_VECTOR_ELT (result, 17, rfilter_params);
+	UNPROTECT(1);
+	if (filter_nparams > 0) {
+	  R_nc_check (nc_inq_var_filter (ncid, varid, NULL, NULL,
+			(unsigned int *) INTEGER (rfilter_params)));
+	}
+#  ifdef NC_ENOFILTER
+      } else if (status == NC_EFILTER || status == NC_ENOFILTER ) {
+#  else
+      } else if (status == NC_EFILTER) {
+#  endif
+	SET_VECTOR_ELT (result, 16, ScalarInteger (NA_INTEGER));
+	SET_VECTOR_ELT (result, 17, ScalarInteger (NA_INTEGER));
+      } else {
+	error (nc_strerror (status));
+      }
+    } else {
+      /* netcdf>=4.7.4 returns NC_EINVAL for non-chunked variables */
+      SET_VECTOR_ELT (result, 16, ScalarInteger (NA_INTEGER));
+      SET_VECTOR_ELT (result, 17, ScalarInteger (NA_INTEGER));
+    }
+#else
+    SET_VECTOR_ELT (result, 16, R_NilValue);
+    SET_VECTOR_ELT (result, 17, R_NilValue);
+#endif
+  }
+
+  UNPROTECT(1);
+  return result;
 }
 
 
@@ -745,9 +758,14 @@ R_nc_put_var (SEXP nc, SEXP var, SEXP start, SEXP count, SEXP data,
   const void *buf;
   double scale, add, *scalep=NULL, *addp=NULL;
   void *fillp=NULL, *minp=NULL, *maxp=NULL;
+  size_t fillsize;
+
+#ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
+  int storeprop;
   size_t bytes, slots;
   float preemption;
   double bytes_in, slots_in, preempt_in;
+#endif
 
   /*-- Convert arguments to netcdf ids ----------------------------------------*/
   ncid = asInteger (nc);
@@ -759,8 +777,10 @@ R_nc_put_var (SEXP nc, SEXP var, SEXP start, SEXP count, SEXP data,
 
   /*-- Chunk cache options for netcdf4 files ----------------------------------*/
 #ifdef HAVE_NC_GET_VAR_CHUNK_CACHE
-  if (nc_get_var_chunk_cache(ncid, varid,
-                             &bytes, &slots, &preemption) == NC_NOERR) {
+  R_nc_check (nc_inq_var_chunking (ncid, varid, &storeprop, NULL));
+  if (storeprop == NC_CHUNKED) {
+    R_nc_check (nc_get_var_chunk_cache(ncid, varid,
+                                       &bytes, &slots, &preemption));
     bytes_in = asReal (cache_bytes);
     slots_in = asReal (cache_slots);
     preempt_in = asReal (cache_preemption);
@@ -793,7 +813,7 @@ R_nc_put_var (SEXP nc, SEXP var, SEXP start, SEXP count, SEXP data,
   }
 
   /*-- Get fill attributes (if any) -------------------------------------------*/
-  R_nc_miss_att (ncid, varid, inamode, &fillp, &minp, &maxp);
+  fillsize = R_nc_miss_att (ncid, varid, inamode, &fillp, &minp, &maxp);
 
   /*-- Get packing attributes (if any) ----------------------------------------*/
   if (ispack) {
@@ -807,11 +827,12 @@ R_nc_put_var (SEXP nc, SEXP var, SEXP start, SEXP count, SEXP data,
 
   /*-- Write variable to file -------------------------------------------------*/
   if (R_nc_length (ndims, ccount) > 0) {
-    buf = R_nc_r2c (data, ncid, xtype, ndims, ccount, fillp, scalep, addp);
+    buf = R_nc_r2c (data, ncid, xtype, ndims, ccount,
+                    fillsize, fillp, scalep, addp);
     R_nc_check (nc_put_vara (ncid, varid, cstart, ccount, buf));
   }
 
-  RRETURN (R_NilValue);
+  return R_NilValue;
 }
 
 
@@ -838,6 +859,6 @@ R_nc_rename_var (SEXP nc, SEXP var, SEXP newname)
   /*-- Rename the variable ----------------------------------------------------*/
   R_nc_check (nc_rename_var (ncid, varid, cnewname));
 
-  RRETURN(R_NilValue);
+  return R_NilValue;
 }
 
